@@ -12,20 +12,16 @@ static const char configKeyDefaultIdentity[] = "Default Identity";
 
 #include <KEmailAddress> // for static helper functions
 
-#include "kidentitymanagement_debug.h"
+#include "kidentitymanagementcore_debug.h"
 #include <KConfig>
 #include <KConfigGroup>
 #include <KEMailSettings> // for IdentityEntry::fromControlCenter()
 #include <KLocalizedString>
 #include <KSharedConfig>
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-#include <Kdelibs4ConfigMigrator>
-#endif
 #include <kuser.h>
 
 #include <QDBusConnection>
 #include <QHostInfo>
-#include <QList>
 #include <QRandomGenerator>
 
 #include <QRegularExpression>
@@ -33,7 +29,7 @@ static const char configKeyDefaultIdentity[] = "Default Identity";
 
 #include "identitymanageradaptor.h"
 
-namespace KIdentityManagement
+namespace KIdentityManagementCore
 {
 static QString newDBusObjectName()
 {
@@ -54,26 +50,26 @@ static QString newDBusObjectName()
 class IdentityManagerPrivate
 {
 public:
-    IdentityManagerPrivate(KIdentityManagement::IdentityManager *);
+    IdentityManagerPrivate(KIdentityManagementCore::IdentityManager *);
     ~IdentityManagerPrivate();
     void writeConfig() const;
     void readConfig(KConfig *config);
     void createDefaultIdentity();
-    Q_REQUIRED_RESULT QStringList groupList(KConfig *config) const;
+    [[nodiscard]] QStringList groupList(KConfig *config) const;
     void slotIdentitiesChanged(const QString &id);
     KConfig *mConfig = nullptr;
 
-    QVector<Identity> mIdentities;
-    QVector<Identity> shadowIdentities;
+    QList<Identity> mIdentities;
+    QList<Identity> shadowIdentities;
 
     // returns a new Unique Object Identifier
-    Q_REQUIRED_RESULT int newUoid();
+    [[nodiscard]] int newUoid();
 
     bool mReadOnly = true;
-    KIdentityManagement::IdentityManager *const q;
+    KIdentityManagementCore::IdentityManager *const q;
 };
 
-IdentityManagerPrivate::IdentityManagerPrivate(KIdentityManagement::IdentityManager *manager)
+IdentityManagerPrivate::IdentityManagerPrivate(KIdentityManagementCore::IdentityManager *manager)
     : q(manager)
 {
 }
@@ -92,7 +88,7 @@ void IdentityManagerPrivate::writeConfig() const
         (*it).writeConfig(cg);
         if ((*it).isDefault()) {
             // remember which one is default:
-            KConfigGroup general(mConfig, "General");
+            KConfigGroup general(mConfig, QStringLiteral("General"));
             general.writeEntry(configKeyDefaultIdentity, (*it).uoid());
 
             // Also write the default identity to emailsettings
@@ -115,7 +111,7 @@ void IdentityManagerPrivate::readConfig(KConfig *config)
         return; // nothing to be done...
     }
 
-    KConfigGroup general(config, "General");
+    KConfigGroup general(config, QStringLiteral("General"));
     uint defaultIdentity = general.readEntry(configKeyDefaultIdentity, 0);
     bool haveDefault = false;
     QStringList::const_iterator groupEnd = identities.constEnd();
@@ -173,7 +169,7 @@ void IdentityManagerPrivate::createDefaultIdentity()
             if (emailAddress.isEmpty()) {
                 emailAddress = user.loginName();
                 if (!emailAddress.isEmpty()) {
-                    KConfigGroup general(mConfig, "General");
+                    KConfigGroup general(mConfig, QStringLiteral("General"));
                     QString defaultdomain = general.readEntry("Default domain");
                     if (!defaultdomain.isEmpty()) {
                         emailAddress += QLatin1Char('@') + defaultdomain;
@@ -230,16 +226,16 @@ int IdentityManagerPrivate::newUoid()
     // determine the UOIDs of all saved identities
     QList<uint> usedUOIDs;
     usedUOIDs.reserve(mIdentities.count() + (q->hasPendingChanges() ? shadowIdentities.count() : 0));
-    const QVector<Identity>::ConstIterator end(mIdentities.constEnd());
-    for (QVector<Identity>::ConstIterator it = mIdentities.constBegin(); it != end; ++it) {
+    const QList<Identity>::ConstIterator end(mIdentities.constEnd());
+    for (QList<Identity>::ConstIterator it = mIdentities.constBegin(); it != end; ++it) {
         usedUOIDs << (*it).uoid();
     }
 
     if (q->hasPendingChanges()) {
         // add UOIDs of all shadow identities. Yes, we will add a lot of duplicate
         // UOIDs, but avoiding duplicate UOIDs isn't worth the effort.
-        const QVector<Identity>::ConstIterator endShadow(shadowIdentities.constEnd());
-        for (QVector<Identity>::ConstIterator it = shadowIdentities.constBegin(); it != endShadow; ++it) {
+        const QList<Identity>::ConstIterator endShadow(shadowIdentities.constEnd());
+        for (QList<Identity>::ConstIterator it = shadowIdentities.constBegin(); it != endShadow; ++it) {
             usedUOIDs << (*it).uoid();
         }
     }
@@ -254,7 +250,7 @@ int IdentityManagerPrivate::newUoid()
 
 void IdentityManagerPrivate::slotIdentitiesChanged(const QString &id)
 {
-    qCDebug(KIDENTITYMANAGEMENT_LOG) << " KIdentityManagement::IdentityManager::slotIdentitiesChanged :" << id;
+    qCDebug(KIDENTITYMANAGEMENT_LOG) << " KIdentityManagementCore::IdentityManager::slotIdentitiesChanged :" << id;
     const QString ourIdentifier = QStringLiteral("%1/%2").arg(QDBusConnection::sessionBus().baseService(), q->property("uniqueDBusPath").toString());
     if (id != ourIdentifier) {
         mConfig->reparseConfiguration();
@@ -277,15 +273,6 @@ IdentityManager::IdentityManager(bool readonly, QObject *parent, const char *nam
     : QObject(parent)
     , d(new IdentityManagerPrivate(this))
 {
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    static bool triedMigration = false;
-    if (!triedMigration) {
-        triedMigration = true;
-        Kdelibs4ConfigMigrator migrate(QStringLiteral("identitymanager"));
-        migrate.setConfigFiles(QStringList() << QStringLiteral("emailidentities"));
-        migrate.migrate();
-    }
-#endif
     setObjectName(QLatin1String(name));
     new IdentityManagerAdaptor(this);
     QDBusConnection dbus = QDBusConnection::sessionBus();
@@ -315,8 +302,8 @@ IdentityManager::IdentityManager(bool readonly, QObject *parent, const char *nam
             KConfigGroup composerGroup = kmailConf->group(QStringLiteral("Composer"));
             if (composerGroup.hasKey(QStringLiteral("pgp-auto-sign"))) {
                 const bool pgpAutoSign = composerGroup.readEntry(QStringLiteral("pgp-auto-sign"), false);
-                const QVector<Identity>::iterator end = d->mIdentities.end();
-                for (QVector<Identity>::iterator it = d->mIdentities.begin(); it != end; ++it) {
+                const QList<Identity>::iterator end = d->mIdentities.end();
+                for (QList<Identity>::iterator it = d->mIdentities.begin(); it != end; ++it) {
                     it->setPgpAutoSign(pgpAutoSign);
                 }
                 composerGroup.deleteEntry(QStringLiteral("pgp-auto-sign"));
@@ -331,8 +318,8 @@ IdentityManager::IdentityManager(bool readonly, QObject *parent, const char *nam
                 if (defaultDomain.isEmpty()) {
                     defaultDomain = QHostInfo::localHostName();
                 }
-                const QVector<Identity>::iterator end = d->mIdentities.end();
-                for (QVector<Identity>::iterator it = d->mIdentities.begin(); it != end; ++it) {
+                const QList<Identity>::iterator end = d->mIdentities.end();
+                for (QList<Identity>::iterator it = d->mIdentities.begin(); it != end; ++it) {
                     it->setDefaultDomainName(defaultDomain);
                 }
                 generalGroup.deleteEntry(QStringLiteral("Default domain"));
@@ -388,14 +375,14 @@ void IdentityManager::commit()
 
     QList<uint> seenUOIDs;
     seenUOIDs.reserve(d->mIdentities.count());
-    const QVector<Identity>::ConstIterator end = d->mIdentities.constEnd();
-    for (QVector<Identity>::ConstIterator it = d->mIdentities.constBegin(); it != end; ++it) {
+    const QList<Identity>::ConstIterator end = d->mIdentities.constEnd();
+    for (QList<Identity>::ConstIterator it = d->mIdentities.constBegin(); it != end; ++it) {
         seenUOIDs << (*it).uoid();
     }
 
     QList<uint> changedUOIDs;
     // find added and changed identities:
-    for (QVector<Identity>::ConstIterator it = d->shadowIdentities.constBegin(); it != d->shadowIdentities.constEnd(); ++it) {
+    for (QList<Identity>::ConstIterator it = d->shadowIdentities.constBegin(); it != d->shadowIdentities.constEnd(); ++it) {
         const int index = seenUOIDs.indexOf((*it).uoid());
         if (index != -1) {
             uint uoid = seenUOIDs.at(index);
@@ -445,6 +432,21 @@ void IdentityManager::rollback()
     d->shadowIdentities = d->mIdentities;
 }
 
+void IdentityManager::saveIdentity(const Identity &ident)
+{
+    const auto existing = std::find_if(modifyBegin(), modifyEnd(), [ident](const auto &existingIdentity) {
+        return existingIdentity.uoid() == ident.uoid();
+    });
+
+    if (existing != modifyEnd()) {
+        d->shadowIdentities.replace(existing - modifyBegin(), ident);
+    } else {
+        d->shadowIdentities << ident;
+    }
+
+    commit();
+}
+
 bool IdentityManager::hasPendingChanges() const
 {
     return d->mIdentities != d->shadowIdentities;
@@ -670,7 +672,7 @@ Identity &IdentityManager::newFromExisting(const Identity &other, const QString
     return result;
 }
 
-QStringList KIdentityManagement::IdentityManager::allEmails() const
+QStringList KIdentityManagementCore::IdentityManager::allEmails() const
 {
     QStringList lst;
     for (ConstIterator it = begin(); it != end(); ++it) {
@@ -686,7 +688,7 @@ void IdentityManager::createDefaultIdentity(QString &, QString &)
 {
 }
 
-void KIdentityManagement::IdentityManager::slotRollback()
+void KIdentityManagementCore::IdentityManager::slotRollback()
 {
     rollback();
 }
